#!/usr/bin/env python
# -*- coding:Utf-8 -*-

########################################################################
# Création d'une jaquette pour CD MP3

# Copyright 2010 Gérard LE REST <gerard@infonie.fr>    
        
#   This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
                               
#######################################################################
# Importation de fonctions externes:
from Tkinter import *
from Back_Cover0_16 import * # image face arrière du cd (sans le titre)
from tumbnails0_12 import * # face avant de la pochette (sans le titre)
from PIL import Image, ImageTk, ImageFont, ImageDraw
from reportlab.pdfgen import canvas
from os import chdir, popen, getcwd, error, sep, environ, path, mkdir
import Pmw, tkMessageBox, sys, tkSimpleDialog, codecs, locale

# noms des fichiers contenant les icônes (format GIF):
textes =('titre du CD','Face avant de la jaquette','Récupérer les tags','Éditer les tags','Dos de la jaquette','Aperçu','Imprimer')
images =('title.png','icone_image1.png','cd1.png','Gartoon-Gedit-icon.png','icone_image2.png','printer-preview.png','printer.png')

class Application(Tk):
    def __init__(self):
        Tk.__init__(self)           # constructeur de la classe parente
        self.L_devant,self.H_devant=1200,1200 # dimensions de l'image de devant
        self.L_back_cover,self.H_back_cover=1380,1180 # dimensions de l'image de derrière
        self.H_image_reduite=450
        self.interface()
        self.photos=[]  # mémorisation pour éviter l'effacement image (bug ImageTk)
        self.coefficient=72.0/254 # coef pour pdf (résolution=72ppp; 1 pouce=254mm)
        # gestion du dossier personnel
        if 'HOME' in environ:
            self.envHome = environ['HOME']
        elif 'HOMEDRIVE' in environ and 'HOMEPATH' in environ:
            self.envHome = path.join(environ['HOMEDRIVE'],os.environ['HOMEPATH'])
        chdir(self.envHome)
        # effacement du dossier ~/PyCDCover précédent:
        if exists("PyCDCover")==TRUE:
            shutil.rmtree("PyCDCover")
        mkdir ("PyCDCover")
        self.repertoire=self.envHome+sep+"PyCDCover"
                    
    def interface(self):
        """création de l'interface graphique"""
        self.fonctions=[self.titres_CD,self.face_avant,self.tags,self.edition,self.dos,self.apercu,self.imprimer]
        # Création de la barre de menu:
        menu1 = Menu(self)
            # menu fichier:
        fichier = Menu(menu1, tearoff=0)
        menu1.add_cascade(label="Fichier",menu=fichier)
        fichier.add_command(label="Titre", command=self.titres_CD)
        fichier.add_command(label="Face avant", command=self.face_avant)
        tags = Menu(fichier, tearoff=0)
        fichier.add_cascade(label="Tags",menu=tags)
        tags.add_command(label="Créer", command=self.tags)
        tags.add_command(label="Éditer", command=self.edition)
        fichier.add_command(label="Dos", command=self.dos)
        fichier.add_separator()
        fichier.add_command(label="Apercu", command=self.apercu)
        fichier.add_command(label="Imprimer", command=self.imprimer)
        fichier.add_separator()
        fichier.add_command(label="Quitter", command=self.quit)
            # menu aide
        infos = Menu(menu1, tearoff=0)
        menu1.add_cascade(label="Aide",menu=infos)
        infos.add_command(label="A propos",command=self.a_propos)
        # afficher le menu
        self.config(menu=menu1)
        # Création d'un objet <bulle d'aide> (un seul suffit) :
        tip = Pmw.Balloon(self)
        # Création de la barre d'outils:
        toolbar = Frame(self, bd =4)
        toolbar.pack(expand =YES, fill =X)
        # Nombre de boutons à construire : 
        nBou = len(images)
        # Les icônes des boutons sont placées dans un liste de variables
        # persistantes:
        self.photoI =[None]*nBou
        self.rep_programme=sys.path[0]	# répertoire du programme 
        for b in range(nBou):
            # Création de l'icône (objet PhotoImage ImageTk) :
            chdir(self.rep_programme+sep+"images")
            image = Image.open(images[b])
            self.photoI[b] = ImageTk.PhotoImage(image)
            # Création du bouton 
            bou = Button(toolbar, image =self.photoI[b], relief =GROOVE, command=self.fonctions[b])
            bou.pack(side =LEFT)
            # association du bouton 1 avec son texte d'aide (bulle) :
            tip.bind(bou, textes[b])
        # Canevas ou seront insérées les images
        self.can1 = Canvas(self,width=self.H_image_reduite, height=self.H_image_reduite, bg="white")
        self.can1.pack(side=LEFT)
        self.can1.bind("<Double-Button-1>",lambda event : self.zoom('Devant.jpeg','Face avant de la jaquette'))
        L=int(self.H_image_reduite*1.169491)
        self.can2 = Canvas(self,width=L, height=self.H_image_reduite, bg ="white")
        self.can2.pack(side=RIGHT)
        self.can2.bind("<Double-Button-1>",lambda event : self.zoom('Dos.png','Dos de la jaquette'))
        #blocage du dimensionnement de la fenêtre
        self.resizable(width=False, height=False)
        
    def titres_CD(self):
        """création des images des titres horizontal et verticaux"""
        chdir(self.repertoire)
        TITRE=tkSimpleDialog.askstring(title="Titre", prompt="Saisir le nom du CD :",parent=self)    #boite de saisie
        # titre horizontal
        im=Image.new("RGB", (self.L_devant,220), "white")
        im.save("TitreH.png", "png")
        draw = ImageDraw.Draw(im)
        police1 = self.rep_programme+"/FreeSerif.ttf"
        taille=60
        font1 = ImageFont.truetype(police1,taille)
        while (draw.textsize(TITRE,font=font1)[1]<120):
            taille=taille+1
            font1 = ImageFont.truetype(police1,taille)
        x=(self.L_devant-draw.textsize(TITRE,font=font1)[0])/2
        y=60
        draw.text((x,y),TITRE, fill="black",anchor=NW, font=font1)
        # tracé de cinq rectangles = rectangle intérieur épais
        for e in range(4):
            draw.rectangle([(x-10-e,y-10-e),(x+10+draw.textsize(TITRE,font=font1)[0]+e,y+10+draw.textsize(TITRE,font=font1)[1]+e)], outline="black")
        # tracé du rectangle extérieur
        draw.rectangle([(x-16,y-16),(x+10+draw.textsize(TITRE,font=font1)[0]+6,y+10+draw.textsize(TITRE,font=font1)[1]+6)], outline="black")
        im.save("TitreH.png", "png")
        # titre vertical1
        im=Image.new("RGB", (self.H_back_cover,60), "white")
        draw = ImageDraw.Draw(im)
        police1 = self.rep_programme+"/FreeSerif.ttf"
        font1 = ImageFont.truetype(police1,40)
        draw.text((40,5),TITRE, fill="black",anchor=NW, font=font1)
        im=im.rotate(90)
        im.save("TitreV1.png", "png")
        # titre vertical2
        im=im.rotate(180)
        im.save("TitreV2.png", "png")
            
    def face_avant(self):
        """création de l'image de la face avant - Image_thumbnails.png"""
        fa=Thumbnails(150,150)
        fa.creation_images()
        fa.assemblage()
        
    def tags(self):
        """création du fichier tag.txt"""
        self.tg=Back_cover()
        self.tg.fichier()

    def edition(self):
        """édition du fichier de tags"""
        chdir(self.repertoire)
        if exists("tags.txt")==TRUE:
            top_fenetre=Toplevel()  # u"création d'une nouvelle fenêtre"
            top_fenetre.title("tags.txt")
            # blocage dimensionnement fenêtre
            top_fenetre.resizable(width=False, height=False)
            self.zone_texte = Pmw.ScrolledText(top_fenetre,usehullsize = 1,hull_width = 300,
                hull_height = 300,text_padx =10, text_pady =10, text_font='Times 11 normal',
                text_bg ='white')
            self.zone_texte.pack()
##            fichier=codecs.open("tags.txt",'r',locale.getdefaultlocale()[1])   #ouvrir un fichier encode en "encodgae du syst d'exploitation"
            fichier=open("tags.txt",'r')
            lignes=fichier.readlines()     # les lignes du fichier sont en unicode
            fichier.closed  
            for i in lignes:               # ajout des lignes dans le fichier
                self.zone_texte.appendtext(i)
            boutong = Button(top_fenetre, text="Enregistrer", command=self.modif_tags)
            boutong.pack(side=LEFT,padx=2, pady=2)
            boutond = Button(top_fenetre, text="Quitter", command=top_fenetre.destroy)
            boutond.pack(side=RIGHT,padx=2, pady=2)
        else:
            tkMessageBox.showinfo("Info","Le fichier texte tags.txt n'a pas été créé")
    
    def modif_tags(self):
        """enregistrement des modification des tags"""
##        fichier=codecs.open("tags.txt",'w',locale.getdefaultlocale()[1])   #ouvrir un fichier encode en "encodgae du syst d'exploitation"
        fichier=codecs.open("tags.txt",'w','utf8')
        fichier.write(self.zone_texte.get('1.0', 'end'))
        fichier.close
                
    def dos(self):
        """création de Image_Back_Cover.png"""
        try:
            self.tg=Back_cover()
            self.tg.image()
        except (IOError):
            tkMessageBox.showinfo("Info","tags non récupérés")
        
    def apercu(self):
        """création des apercus des couvertures à l'écran"""
        chdir(self.repertoire)
        # création de l'image "face avant de la jaquette"
        affichage=0 # nbre d'affichage de la boite de dialogue "titre manquant"
        if exists("Image_thumbnails.jpeg")==TRUE and exists("TitreH.png")==TRUE:
            self.Devant=self.face("Image_thumbnails.jpeg",self.L_devant,self.H_devant)
            self.Devant.save("Devant.jpeg","jpeg")
            # création et insertion de l'apercu de l'image "Devant.png"
                # redimensionnement
            im=Image.open("Devant.jpeg")
            out=im.resize((self.H_image_reduite,self.H_image_reduite))  
                # insertion de l'aprecu dans le label
            photo1 = ImageTk.PhotoImage(out)
            self.can1.create_image(0,0,anchor =NW,image=photo1)
            self.photos.append(photo1)    # mémorisation pour éviter l'effacement image (bug ImageTk)
        if exists("Image_thumbnails.jpeg")==FALSE:
            tkMessageBox.showinfo("info","face avant de la jaquette non créée")
        if exists("TitreH.png")==FALSE:
            tkMessageBox.showinfo("Info","titre manquant")
            affichage=1
        if exists("Image_Back_Cover.png")==TRUE and exists("TitreH.png")==TRUE:
            # création de l'image "dos de la jaquette"
            self.Dos=self.face("Image_Back_Cover.png",self.L_back_cover,self.H_back_cover)
            self.Dos.save("Dos.png","png")
            # création et insertion de l'apercu de l'image "dos"
                # redimensionnement    
            im=Image.open("Dos.png")
            L,H=im.size
            Lf,Hf=float(L),float(H)
            rapport=Lf/Hf
            out=im.resize((int(self.H_image_reduite*rapport),self.H_image_reduite))  
                # insertion de l'aprecu dans le canevas
            photo2 = ImageTk.PhotoImage(out)
            self.can2.create_image(0,0,anchor = NW,image=photo2)
            self.photos.append(photo2)    # mémorisation pour éviter l'effacement image (bug ImageTk)
        if exists("Image_Back_Cover.png")==FALSE:
           tkMessageBox.showinfo("Info","Pour afficher le dos de la jaquette, il faut récupérer les tags et créer l'image correspondante") 
        if (affichage==0) and (exists("TitreH.png")==FALSE):
            tkMessageBox.showinfo("Info","titre manquant")
        
    def face(self,image_originale,largeur, hauteur):
        """création des images (face avant et dos sans titres verticaux) à imprimer"""
        chdir(self.repertoire)
        im=Image.new("RGB", (largeur,hauteur), "white")
        draw = ImageDraw.Draw(im)
        into1=Image.open("TitreH.png")
        L,H=into1.size
        marge_x=(largeur-L)/2
        im.paste(into1,(0+marge_x,0,L+marge_x,H))
        im2=Image.open(image_originale)
        # redimensionnement
        L,H=im2.size
        Lf,Hf=float(L),float(H)
        rapport=Hf/Lf        # rapport hauteur/longueur de l'image
        if (L==self.L_back_cover) and (H<=930):
        	out=im2 #image Image_Back_Cover.png
        elif Hf>Lf:
            L_modifie=int(960/rapport)
            out=im2.resize((L_modifie,960))
        else:
            H_modifie=int(960*rapport)
            out=im2.resize((960,H_modifie))
        L,H=out.size
        marge_x=(largeur-L)/2
        im.paste(out,(0+marge_x,220,L+marge_x,H+220))
        return im
            
    def zoom(self,image,titre):
        """zoom de l'aperçu"""
        chdir(self.repertoire)
        if exists(image)==TRUE:
            fen_zoom=Toplevel()
            fen_zoom.title(titre)
            H,L=self.winfo_screenheight(), self.winfo_screenwidth() # résolution de l'écran
            Sc=Pmw.ScrolledCanvas(fen_zoom,
                 usehullsize =1, hull_width =(L-60), hull_height=(H-60),
                 borderframe =1, borderframe_borderwidth =3)
        # configuration des barres de défilement:         
            Sc.configure(vscrollmode ='dynamic', hscrollmode ='dynamic')        
            Sc.pack(padx =5, pady =5, expand =YES, fill =BOTH)
            # Ajout de l'image
            Sc.can=Sc.interior()        # accès au composant canevas
            im=Image.open(image)
            photo_dos=ImageTk.PhotoImage(im)   
            Sc.can.create_image(0,0,image=photo_dos)
            self.photos.append(photo_dos) # mémorisation - bug ImageTk    
            Sc.resizescrollregion()
        else:
            tkMessageBox.showinfo("Info","L'image de la face n'a pas été créée.")
        
    def imprimer(self):
        """impression de la jaquette"""
        chdir(self.repertoire)
        self.creation_gabarit()
        if exists("image_impression.pdf")==TRUE:
            popen('lp image_impression.pdf')
            
    def creation_gabarit(self):
        """creation d'un gabarit d'impression: deux images avant et arriere + titres vericaux + trace decoupage dans une page PDF"""
        try:
            c=canvas.Canvas("image_impression.pdf") # instantation d'un objet canevas reportlab
            c.setLineWidth(1)
            c.setDash(6,3)
            # création des lignes en pointillés de découpage
                # lignes verticales
            origines=[(300*self.coefficient,0),(360*self.coefficient,0),(1560*self.coefficient,0),(1740*self.coefficient,0),(1800*self.coefficient,0)]
            H=2970*self.coefficient
            for i in origines:
                xorigine,yorigine=i[0],i[1]
                c.line(xorigine,yorigine,xorigine,yorigine+H)
            # lignes horizontales
            origines=[(0,2860*self.coefficient),(0,1660*self.coefficient),(0,480*self.coefficient)]
            L=2100*self.coefficient
            for i in origines:
                xorigine,yorigine=i[0],i[1]
                c.line(xorigine,yorigine,xorigine+L,yorigine)      
            # insertion de l'image Devant.png
            xorigine,yorigine=360*self.coefficient,1660*self.coefficient
            c.drawImage("Devant.jpeg",xorigine,yorigine, width=self.L_devant*self.coefficient,height=self.H_devant*self.coefficient,mask=None)
            # insertion de l'image Dos.png
            into=Image.open("Dos.png")
            L,H=into.size
            xorigine,yorigine=360*self.coefficient,480*self.coefficient
            c.drawImage("Dos.png",xorigine,yorigine, width=self.L_back_cover*self.coefficient,height=self.H_back_cover*self.coefficient,mask=None)
            # insertion des titres verticaux
            xorigine,yorigine=300*self.coefficient,480*self.coefficient
            c.drawImage("TitreV1.png",xorigine,yorigine, width=60*self.coefficient,height=self.H_back_cover*self.coefficient,mask=None)
            xorigine,yorigine=1740*self.coefficient,480*self.coefficient
            c.drawImage("TitreV2.png",xorigine,yorigine, width=60*self.coefficient,height=self.H_back_cover*self.coefficient,mask=None)
            # tracé des rectangles en traits pleins
            c.setDash(1,0)
            c.rect(360*self.coefficient,1660*self.coefficient,self.L_devant*self.coefficient,self.H_devant*self.coefficient)
            c.rect(300*self.coefficient,480*self.coefficient,60*self.coefficient,self.H_back_cover*self.coefficient)
            c.rect(1740*self.coefficient,480*self.coefficient,60*self.coefficient,self.H_back_cover*self.coefficient)
            c.rect(360*self.coefficient,480*self.coefficient,self.L_back_cover*self.coefficient,self.H_back_cover*self.coefficient)
            c.showPage()    # sauvegarde de la page courante
            c.save()    # enregistrement du fichier et fermeture du canveas reportlab
        except IOError:
            tkMessageBox.showinfo(title="Info",message="Toutes les images de la jaquette n'ont pas été créées")
           
    def a_propos(self):
        """fenêtre indiquant l'auteur et le type de licence"""
        fen_propos=Toplevel()
        fen_propos.title("À propos de: ")
        fram1=Frame(fen_propos,padx=5,pady=5)
        fram1.pack()
        Message(fram1, width=200, aspect=50, justify=CENTER,pady=5, 
                text=" PyCDCover\n\nCopyrigth G LE REST Mars 2010.\n"\
                "Licence GNU-GPL\n\n http://blogclisam.chez.com/").pack()
        bou = Button(fram1,text ="Quitter", 
                     command =fen_propos.destroy).pack()
        #blocage du dimensionnement de la fenêtre
        fen_propos.resizable(width=False, height=False)


#########################################################################      

if __name__ == "__main__":
    app = Application()
    app.title('PyCDCover')
    app.mainloop()
